@@ -2,7 +2,7 @@
 
 # crosser.sh: Generic toolchain builder.
 #
-# (c) 2008-2011 Marko Lindqvist
+# (c) 2008-2012 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -228,6 +228,13 @@ build_glibc() {
       log_error "Failed to create dummy stubs.h"
       return 1
     fi
+
+    if ! cp "$CROSSER_MAINDIR/scripts/dummypthread.h" \
+            "$SYSPREFIX/usr/include/pthread.h"
+    then
+      log_error "Failed to copy dummy pthread.h"
+      return 1
+    fi
   fi
 }
 
@@ -411,7 +418,7 @@ link_host_command() {
 # 1 - Failure
 setup_host_commands() {
   # Absolutely required commands
-  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo install whoami depmod wget pod2man msgfmt sh glib-genmarshal hostname dnsdomainname mktemp"
+  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which tee sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 xz flex makeinfo install whoami depmod wget pod2man msgfmt sh hostname dnsdomainname mktemp"
   # Usefull commands
   HOST_COMMANDS_TRY="dpkg-source md5sum gpg gpgv sha1sum sha256sum gzip gunzip patch quilt bash getopt"
 
@@ -494,6 +501,7 @@ dummy_glibc_objects() {
 #
 prepare_binutils_src() {
   if ! unpack_component binutils     $VERSION_BINUTILS              ||
+     ! patch_src binutils-$BASEVER_BINUTILS binutils_LIBP_for_build ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
         (  patch_src binutils-$BASEVER_BINUTILS binutils_makeinfo &&
            cd "$CROSSER_SRCDIR/binutils-$BASEVER_BINUTILS" && autoconf )) ||
@@ -522,7 +530,9 @@ prepare_gcc_src() {
   fi
 
   if ! (! cmp_versions $VERSION_GCC 4.3.1 ||
-        patch_src gcc-$VERSION_GCC gcc_cldconf )
+        patch_src gcc-$VERSION_GCC gcc_cldconf ) ||
+     ! (is_smaller_version $VERSION_MPFR 3.1.0 ||
+        patch_src gcc-$VERSION_GCC gcc_mpfr310_mpc )
   then
     log_error "GCC patching failed"
     exit 1
@@ -538,6 +548,24 @@ prepare_gcc_src() {
   fi
 }
 
+# Basic preparation of libgpg-error source tree
+#
+prepare_gpg_error_src() {
+  if ! unpack_component libgpg-error $VERSION_GPGERROR
+  then
+    log_error "Libgcpg-error unpacking failed"
+    exit 1
+  fi
+
+  if ! patch_src libgpg-error-$VERSION_GPGERROR gpgerror_LIBP_for_build ||
+     ! autogen_component libgpg-error $VERSION_GPGERROR                 \
+       "aclocal automake autoconf"
+  then
+    log_error "Failed to prepare libgpg-error sources"
+    exit 1
+  fi
+}
+
 # Basic preparation of libgcrypt source tree
 #
 prepare_libgcrypt_src() {
@@ -636,9 +664,9 @@ export PKG_CONFIG_PATH=$NATIVE_PREFIX/lib/pkgconfig
 NO_VEND=$(. $CROSSER_MAINDIR/setups/native.sh ; echo $TMP_ARCH-$TMP_OS)
 if test "x$LIBRARY_PATH" = "x"
 then
-  CROSSER_NAT_LIBP="/usr/lib/$NO_VEND"
+  export CROSSER_NAT_LIBP="/usr/lib/$NO_VEND"
 else
-  CROSSER_NAT_LIBP="$LIBRARY_PATH:/usr/lib/$NO_VEND"
+  export CROSSER_NAT_LIBP="$LIBRARY_PATH:/usr/lib/$NO_VEND"
 fi
 CROSSER_NAT_INCP="/usr/include/$NO_VEND"
 
@@ -664,9 +692,22 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      ! build_for_host   automake automake-$VERSION_AUTOMAKE   ||
      ! unpack_component Python3  $VERSION_PYTHON3             ||
      ! build_for_host   Python3  Python-$VERSION_PYTHON3      ||
-     ! unpack_component gtk-doc  $VERSION_GTK_DOC             ||
-     ! build_for_host   gtk-doc  gtk-doc-$VERSION_GTK_DOC         \
-       "--disable-scrollkeeper"                                   ||
+     ! unpack_component Python2  $VERSION_PYTHON2             ||
+     ! build_for_host   Python2  Python-$VERSION_PYTHON2      ||
+     ! prepare_gpg_error_src                                        ||
+     ! build_for_host   libgpg-error libgpg-error-$VERSION_GPGERROR ||
+     ! prepare_libgcrypt_src                                        ||
+     ! build_for_host   libgcrypt libgcrypt-$VERSION_LIBGCRYPT      \
+         "--disable-asm"                                            ||
+     ! unpack_component libxml2 $VERSION_LIBXML2                    ||
+     ! build_for_host   libxml2 libxml2-$VERSION_LIBXML2            \
+       "--without-python"                                           ||
+     ! unpack_component libxslt    $VERSION_LIBXSLT                 ||
+     ! build_for_host   libxslt    libxslt-$VERSION_LIBXSLT         \
+       "--without-python"                                           ||
+     ! unpack_component gtk-doc  $VERSION_GTK_DOC                   ||
+     ! build_for_host   gtk-doc  gtk-doc-$VERSION_GTK_DOC           \
+       "--disable-scrollkeeper"                                     ||
      ! unpack_component libffi     $VERSION_FFI                   ||
      ! build_for_host   libffi     libffi-$VERSION_FFI            ||
      ! unpack_component glib       $VERSION_GLIB                  ||
@@ -675,18 +716,15 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
         patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
      ! build_for_host   pkg-config pkg-config-$VERSION_PKG_CONFIG ||
-     ! unpack_component          libgpg-error $VERSION_GPGERROR     ||
-     ! build_for_host   libgpg-error libgpg-error-$VERSION_GPGERROR ||
-     ! prepare_libgcrypt_src                                        ||
-     ! build_for_host   libgcrypt libgcrypt-$VERSION_LIBGCRYPT      \
-         "--disable-asm"                                            ||
-     ! unpack_component libxslt    $VERSION_LIBXSLT               ||
-     ! build_for_host   libxslt    libxslt-$VERSION_LIBXSLT       ||
-     ! prepare_binutils_src                                   ||
-     ! build_for_host binutils binutils-$BASEVER_BINUTILS     \
-     "--with-tls --enable-stage1-languages=all"               ||
-     ! prepare_gcc_src                                        ||
-     ! log_write 1 "-Building native gcc can take hours!-"    ||
+     ! unpack_component util-macros $VERSION_XORG_UMACROS             ||
+     ! build_for_host   util-macros util-macros-$VERSION_XORG_UMACROS ||
+     ! unpack_component xtrans      $VERSION_XORG_XTRANS              ||
+     ! build_for_host   xtrans      xtrans-$VERSION_XORG_XTRANS       ||
+     ! prepare_binutils_src                                         ||
+     ! build_for_host binutils binutils-$BASEVER_BINUTILS           \
+     "--with-tls --enable-stage1-languages=all --with-sysroot"      ||
+     ! prepare_gcc_src                                              ||
+     ! log_write 1 "-Building native gcc can take hours!-"          ||
      ! LIBRARY_PATH="$CROSSER_NAT_LIBP" build_for_host gcc gcc-$VERSION_GCC                    \
      "--enable-languages=c,c++ --disable-multilib --with-tls"
   then
@@ -884,7 +922,9 @@ then
           patch_src $LIBCDIR/ports glibc_ports_arm_pageh_inc)    ||
        ! patch_src $LIBCDIR/ports glibc_ports_arm_tlsinc         ||
        ! (! cmp_versions $LIBCVER 2.9 ||
-          patch_src $LIBCDIR/ports glibc_upstream_arm_sigsetjmp)
+          patch_src $LIBCDIR/ports glibc_upstream_arm_sigsetjmp) ||
+       ! (is_smaller_version $LIBCVER 2.14 ||
+          patch_src $LIBCDIR glibc_cpuidh)
     then
       crosser_error "$LIBCNAME patching failed"
       exit 1
@@ -914,7 +954,7 @@ then
     STEP="chain(2)"
 
     if ! build_with_native_compiler gcc gcc-$VERSION_GCC \
-          "--disable-multilib --enable-languages=c --with-tls --with-sysroot=$SYSPREFIX --disable-threads --disable-libssp --disable-libgomp --disable-libmudflap"
+          "--disable-multilib --enable-languages=c --with-tls --with-sysroot=$SYSPREFIX --disable-threads --disable-libssp --disable-libgomp --disable-libmudflap --disable-libquadmath"
     then
       crosser_error "Failed to build phase 2 cross-compiler"
       exit 1
@@ -951,7 +991,7 @@ fi   # STEP_CHAIN
 export CCACHE_DIR="$CROSSER_DST_PFX/.ccache"
 
 unset PKG_CONFIG_PATH
-export PKG_CONFIG_LIBDIR="$CROSSER_IM_PFX/lib/pkgconfig:$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
+export PKG_CONFIG_LIBDIR="$CROSSER_IM_PFX/lib/pkgconfig:$CROSSER_IM_PFX/share/pkgconfig:$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
 
 PYTHON3_SUBDIR="python$(echo $VERSION_PYTHON3 | sed 's/\./ /g' |
 ( read MAJ MIN PATCH ; echo $MAJ.$MIN ))"
@@ -971,9 +1011,8 @@ then
   fi
 
   # Some patches work only when compiling for Windows target and
-  # thus are used in libstack.sh only
-  if ! patch_src zlib               zlib_cctest               ||
-     ! patch_src zlib               zlib_seeko
+  # thus are used in dllstack.sh only
+  if ! patch_src zlib               zlib_seeko-1.2.6-2
   then
     crosser_error "Baselib patching failed"
     exit 1
@@ -997,8 +1036,8 @@ then
     # If native step already executed, libxslt already unpacked
     if test "x$STEP_NATIVE" != "xyes"
     then
-      if ! unpack_component libgpg-error $VERSION_GPGERROR ||
-         ! prepare_libgcrypt_src                           ||
+      if ! prepare_gpg_error_src                     ||
+         ! prepare_libgcrypt_src                     ||
          ! unpack_component libxslt $VERSION_LIBXSLT
       then
         crosser_error "Unpacking of baselib source packages failed"
@@ -1008,7 +1047,7 @@ then
 
     if ! unpack_component libxml2 $VERSION_LIBXML2                        ||
        ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       \
-         "--prefix=$CROSSER_IM_PFX" "" "/"                                ||
+         "--prefix=$CROSSER_IM_PFX --without-python" "" "/"               ||
        ! build_with_cross_compiler libgpg-error \
          libgpg-error-$VERSION_GPGERROR \
          "--prefix=$CROSSER_IM_PFX" "" "/"                                ||
@@ -1017,7 +1056,7 @@ then
          "" "/" ||
        ! link_im_host_prog libgcrypt-config ||
        ! build_with_cross_compiler libxslt   libxslt-$VERSION_LIBXSLT     \
-         "--prefix=$CROSSER_IM_PFX --with-libxml-prefix=$CROSSER_IM_PFX"  \
+         "--prefix=$CROSSER_IM_PFX --with-libxml-prefix=$CROSSER_IM_PFX --without-python" \
          "" "/" ||
        ! unpack_component jpeg $VERSION_JPEG "" "jpegsrc.v${VERSION_JPEG}" ||
        ! build_with_cross_compiler jpeg      jpeg-$VERSION_JPEG           \
@@ -1030,13 +1069,14 @@ then
     STEP="bl(tgt)"
     STEPADD="  "
 
-    if ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       ||
+    if ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       \
+         "--without-python"                                               ||
        ! build_with_cross_compiler libgpg-error \
          libgpg-error-$VERSION_GPGERROR                                   ||
        ! build_with_cross_compiler libgcrypt libgcrypt-$VERSION_LIBGCRYPT \
          "--with-gpg-error-prefix=$CROSSER_IM_PFX --disable-asm"          ||
        ! build_with_cross_compiler libxslt   libxslt-$VERSION_LIBXSLT     \
-         "--with-libxml-prefix=$CROSSER_IM_PFX"                           ||
+         "--with-libxml-prefix=$CROSSER_IM_PFX --without-python"          ||
        ! build_with_cross_compiler jpeg      jpeg-$VERSION_JPEG
     then
       crosser_error "Baselib target build failed"
@@ -1054,16 +1094,15 @@ then
   then
     log_write 1 "Step xorg not available for $LIBC_MODE based builds, skipping"
   else
-    if ! unpack_component xproto $VERSION_XORG_XPROTO                          ||
-       ! build_with_cross_compiler xproto xproto-$VERSION_XORG_XPROTO          ||
-       ! unpack_component xextproto $VERSION_XORG_XEXTPROTO                    ||
-       ! build_with_cross_compiler xextproto xextproto-$VERSION_XORG_XEXTPROTO ||
-       ! unpack_component xtrans $VERSION_XORG_XTRANS                          ||
-       ! build_with_cross_compiler xtrans xtrans-$VERSION_XORG_XTRANS          ||
-       ! unpack_component libpthread-stubs $VERSION_PTHREAD_STUBS              ||
-       ! build_with_cross_compiler libpthread-stubs libpthread-stubs-$VERSION_PTHREAD_STUBS ||
-       ! unpack_component libXau    $VERSION_XORG_LIBXAU                            ||
-       ! build_with_cross_compiler libXau libXau-$VERSION_XORG_LIBXAU
+    if ! unpack_component util-macros $VERSION_XORG_UMACROS                      ||
+       ! build_with_cross_compiler util-macros util-macros-$VERSION_XORG_UMACROS \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                       ||
+       ! unpack_component xproto $VERSION_XORG_XPROTO                            ||
+       ! build_with_cross_compiler xproto xproto-$VERSION_XORG_XPROTO            ||
+       ! unpack_component xextproto $VERSION_XORG_XEXTPROTO                      ||
+       ! build_with_cross_compiler xextproto xextproto-$VERSION_XORG_XEXTPROTO   ||
+       ! unpack_component libpthread-stubs $VERSION_PTHREAD_STUBS                ||
+       ! build_with_cross_compiler libpthread-stubs libpthread-stubs-$VERSION_PTHREAD_STUBS
     then
       crosser_error "Xorg build failed"
       exit 1
@@ -1072,8 +1111,17 @@ then
     STEP="xorg(im)"
     STEPADD=" "
 
-    if ! unpack_component xcb-proto $VERSION_XCB_PROTO                         ||
+    if ! unpack_component          xtrans $VERSION_XORG_XTRANS                 ||
+       ! build_with_cross_compiler xtrans xtrans-$VERSION_XORG_XTRANS          \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                     ||
+       ! unpack_component          libXau $VERSION_XORG_LIBXAU                 ||
+       ! build_with_cross_compiler libXau libXau-$VERSION_XORG_LIBXAU          \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                     ||
+       ! unpack_component          xcb-proto $VERSION_XCB_PROTO                ||
        ! build_with_cross_compiler xcb-proto xcb-proto-$VERSION_XCB_PROTO      \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                     ||
+       ! unpack_component          libxcb    $VERSION_LIBXCB                   ||
+       ! build_with_cross_compiler libxcb    libxcb-$VERSION_LIBXCB            \
          "--prefix=$CROSSER_IM_PFX" "" "/"
     then
       crosser_error "Xorg intermediate part build failed"
@@ -1083,7 +1131,21 @@ then
     STEP="xorg(tgt)"
     STEPADD=""
 
-    if ! build_with_cross_compiler xcb-proto xcb-proto-$VERSION_XCB_PROTO
+    if ! build_with_cross_compiler xtrans    xtrans-$VERSION_XORG_XTRANS          ||
+       ! build_with_cross_compiler xcb-proto xcb-proto-$VERSION_XCB_PROTO         ||
+       ! build_with_cross_compiler libXau    libXau-$VERSION_XORG_LIBXAU          ||
+       ! build_with_cross_compiler libxcb    libxcb-$VERSION_LIBXCB               ||
+       ! unpack_component          kbproto   $VERSION_XORG_KBPROTO                ||
+       ! build_with_cross_compiler kbproto   kbproto-$VERSION_XORG_KBPROTO        ||
+       ! unpack_component          inputproto $VERSION_XORG_INPUTPROTO            ||
+       ! build_with_cross_compiler inputproto inputproto-$VERSION_XORG_INPUTPROTO ||
+       ! unpack_component          libX11     $VERSION_XORG_LIBX11                ||
+       ! patch_src libX11-$VERSION_XORG_LIBX11 "x11_LIBP_for_build"               ||
+       ! autogen_component         libX11     $VERSION_XORG_LIBX11                \
+         "libtoolize aclocal automake autoconf"                                   ||
+       ! build_with_cross_compiler libX11     libX11-$VERSION_XORG_LIBX11         ||
+       ! unpack_component          libXext    $VERSION_XORG_LIBXEXT               ||
+       ! build_with_cross_compiler libXext    libXext-$VERSION_XORG_LIBXEXT
     then
       crosser_error "Xorg build failed"
       exit 1
@@ -1103,6 +1165,9 @@ then
     GLIB_VARS="$(read_configure_vars glib)"
     log_write 4 "Glib variables: $GLIB_VARS"
 
+    GDK_PB_VARS="$(read_configure_vars gdk-pixbuf)"
+    log_write 4 "Gdk-pixbuf variables: $GDK_PB_VARS"
+
     # If native step already executed, libffi and glib already unpacked
     if test "x$STEP_NATIVE" != "xyes"
     then
@@ -1121,8 +1186,10 @@ then
        ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
          "--prefix=$CROSSER_IM_PFX $GLIB_VARS" "" "/"                                    ||
        ! unpack_component          freetype      $VERSION_FREETYPE                       ||
+       ! patch_src freetype-$VERSION_FREETYPE freetype_LIBP_for_build                    ||
+       ! autogen_component         freetype      $VERSION_FREETYPE                       ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
-         "--prefix=$CROSSER_IM_PFX" "" "/" ||
+         "--prefix=$CROSSER_IM_PFX --without-bzip2" "" "/"                               ||
        ! unpack_component          pixman        $VERSION_PIXMAN                         ||
        ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
          "--prefix=$CROSSER_IM_PFX --disable-gtk" "" "/" ||
@@ -1131,6 +1198,9 @@ then
          "--prefix=$CROSSER_IM_PFX" "" "/" ||
        ! unpack_component          fontconfig    $VERSION_FONTCONFIG                     ||
        ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_cross                       ||
+       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_LIBP_for_build              ||
+       ! autogen_component         fontconfig    $VERSION_FONTCONFIG                     \
+         "libtoolize aclocal automake autoconf"                                          ||
        ! build_with_cross_compiler fontconfig                                            \
           fontconfig-$VERSION_FONTCONFIG                                                 \
           "--prefix=$CROSSER_IM_PFX --with-freetype-config=$CROSSER_IM_PFX/bin/freetype-config --with-arch=$TARGET" \
@@ -1144,10 +1214,11 @@ then
     STEPADD=" "
     if ! build_with_cross_compiler libffi        libffi-$VERSION_FFI                     ||
        ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
-         "--prefix=/usr $GLIB_VARS" ||
-       ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              ||
+         "--prefix=/usr $GLIB_VARS"                                                      ||
+       ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
+         "--without-bzip2"                                                               ||
        ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
-         "--prefix=/usr --disable-gtk" ||
+         "--prefix=/usr --disable-gtk"                                                   ||
        ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    ||
        ! build_with_cross_compiler fontconfig                                            \
           fontconfig-$VERSION_FONTCONFIG                                                 \
@@ -1159,12 +1230,17 @@ then
     fi
 
     STEP="gtk"
-    STEPADD="     "
+    STEPADD="      "
     if ! unpack_component          atk           $VERSION_ATK                            ||
        ! build_with_cross_compiler atk           atk-$VERSION_ATK                        \
-         "--disable-glibtest" ||
+         "--prefix=/usr --disable-glibtest" ||
        ! unpack_component          tiff          $VERSION_TIFF                           ||
-       ! build_with_cross_compiler tiff          tiff-$VERSION_TIFF
+       ! build_with_cross_compiler tiff          tiff-$VERSION_TIFF                      ||
+       ! unpack_component          gdk-pixbuf    $VERSION_GDK_PIXBUF                     ||
+       ! build_with_cross_compiler gdk-pixbuf    gdk-pixbuf-$VERSION_GDK_PIXBUF          \
+         "$GDK_PB_VARS"                                                                  ||
+       ! unpack_component          cairo         $VERSION_CAIRO                          ||
+       ! build_with_cross_compiler cairo         cairo-$VERSION_CAIRO
     then
       crosser_error "gtk+ chain build failed"
       exit 1
@@ -1175,7 +1251,7 @@ fi
 if test "x$STEP_SDL" = "xyes"
 then
   STEP="sdl"
-  STEPADD="     "
+  STEPADD="      "
 
   if test "x$E_GLIBC" != "xyes"
   then
@@ -1205,7 +1281,7 @@ then
     fi
 
     STEP="sdl(im)"
-    STEPADD=" "
+    STEPADD="  "
     if ! build_with_cross_compiler SDL       SDL-$VERSION_SDL             \
          "--prefix=$CROSSER_IM_PFX" "" "/"
     then
@@ -1214,7 +1290,7 @@ then
     fi
 
     STEP="sdl(tgt)"
-    STEPADD=""
+    STEPADD=" "
     if ! build_with_cross_compiler SDL       SDL-$VERSION_SDL
     then
       crosser_error "Target SDL build failed"
@@ -1222,7 +1298,7 @@ then
     fi
 
     STEP="sdl"
-    STEPADD="     "
+    STEPADD="      "
     if ! unpack_component          SDL_image $VERSION_SDL_IMAGE           ||
        ! build_with_cross_compiler SDL_image SDL_image-$VERSION_SDL_IMAGE
     then
