@@ -2,7 +2,7 @@
 
 # dllstack.sh: Cross-compile set of libraries for Windows target.
 #
-# (c) 2008-2011 Marko Lindqvist
+# (c) 2008-2012 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 #
@@ -103,7 +103,7 @@ build_component_host()
 # $2 - Component
 # $3 - Version, "0" to indicate that there isn't package to build after all
 # $4 - Extra configure options
-# $5 - Native ('native')
+# $5 - Build type ('native' | 'windres')
 build_component_full()
 {
   log_packet "$1"
@@ -132,15 +132,20 @@ build_component_full()
   cd "$BUILDDIR"
   SRCDIR="$CROSSER_SRCDIR/$SUBDIR"
 
-  if test "x$5" != "xnative"
+  if test "x$5" = "xnative"
   then
-    CONFOPTIONS="--prefix=$DLLSPREFIX --build=$BUILD --host=$TARGET --target=$TARGET $4"
-    export CPPFLAGS="-isystem $DLLSPREFIX/include -isystem $TGT_HEADERS $TGT_MARCH $USER_CPPFLAGS"
-    export LDFLAGS="-L$DLLSPREFIX/lib $USER_LDFLAGS"
-  else
     CONFOPTIONS="--prefix=$NATIVE_PREFIX $4"
     unset CPPFLAGS
     unset LDFLAGS
+  elif test "x$5" = "xwindres"
+  then
+    CONFOPTIONS="--prefix=$DLLSPREFIX --build=$BUILD --host=$TARGET --target=$TARGET $4"
+    unset CPPFLAGS
+    export LDFLAGS="-L$DLLSPREFIX/lib $USER_LDFLAGS"
+  else
+    CONFOPTIONS="--prefix=$DLLSPREFIX --build=$BUILD --host=$TARGET --target=$TARGET $4"
+    export CPPFLAGS="-isystem $DLLSPREFIX/include -isystem $TGT_HEADERS $TGT_MARCH $USER_CPPFLAGS"
+    export LDFLAGS="-L$DLLSPREFIX/lib $USER_LDFLAGS"
   fi
 
   if test -x "$SRCDIR/configure"
@@ -460,6 +465,7 @@ if ! unpack_component     autoconf   $VERSION_AUTOCONF      ||
    ! build_component_host automake   $VERSION_AUTOMAKE      ||
    ! free_component       automake   $VERSION_AUTOMAKE "host-automake" ||
    ! unpack_component     libtool    $VERSION_LIBTOOL       ||
+   ! patch_src libtool-$VERSION_LIBTOOL libtool_bash        ||
    ! build_component_host libtool    $BASEVER_LIBTOOL       ||
    ! free_build           "host-libtool"                    ||
    ! unpack_component     libffi     $VERSION_FFI           ||
@@ -485,15 +491,19 @@ fi
 
 export PKG_CONFIG_LIBDIR="$DLLSPREFIX/lib/pkgconfig"
 
+if cmp_versions $VERSION_SQLITE 3.7.10
+then
+  SQL_VERSTR="3071000"
+fi
+
 if ! build_component   libtool    $BASEVER_LIBTOOL                   ||
    ! free_component    libtool    $BASEVER_LIBTOOL "libtool"         ||
    ! unpack_component  libiconv   $VERSION_ICONV                     ||
    ! build_component   libiconv   $VERSION_ICONV                     ||
    ! free_component    libiconv   $VERSION_ICONV "libiconv"          ||
    ! unpack_component  zlib       $VERSION_ZLIB                      ||
-   ! patch_src zlib               zlib_cctest                        ||
-   ! patch_src zlib               zlib_seeko                         ||
-   ! patch_src zlib               zlib_nolibc                        ||
+   ! patch_src zlib               zlib_seeko-1.2.6-2                 ||
+   ! patch_src zlib               zlib_nolibc-1.2.6-2                ||
    ! patch_src zlib               zlib_dllext                        ||
    ! build_zlib        zlib       $VERSION_ZLIB                      ||
    ! free_src          zlib       $VERSION_ZLIB                      ||
@@ -503,9 +513,16 @@ if ! build_component   libtool    $BASEVER_LIBTOOL                   ||
    ! patch_src bzip2-$VERSION_BZIP2 bzip2_winapi                     ||
    ! build_bzip2       bzip2      $VERSION_BZIP2                     ||
    ! free_src          bzip2      $VERSION_BZIP2                     ||
+   ! unpack_component  xz         $VERSION_XZ                        ||
+   ! build_component_full xz xz   $VERSION_XZ "" "windres"           ||
+   ! free_component    xz         $VERSION_XZ "xz"                   ||
    ! unpack_component  curl       $VERSION_CURL                      ||
    ! build_component   curl       $VERSION_CURL                      ||
    ! free_component    curl       $VERSION_CURL "curl"               ||
+   ! unpack_component  sqlite     $VERSION_SQLITE                    \
+     "" "sqlite-autoconf-${SQL_VERSTR}"                              ||
+   ! build_component_full sqlite sqlite-autoconf $SQL_VERSTR         ||
+   ! free_component    sqlite-autoconf $SQL_VERSTR "sqlite"          ||
    ! unpack_component  libpng     $VERSION_PNG                       ||
    ! patch_src libpng-$VERSION_PNG png_symbol_prefix                 ||
    ! autogen_component libpng     $VERSION_PNG                       ||
@@ -593,11 +610,8 @@ then
 fi
 
 if ! unpack_component  freetype   $VERSION_FREETYPE               ||
-   ! ( is_minimum_version $VERSION_FREETYPE 2.3.6 ||
-       patch_src freetype-$VERSION_FREETYPE freetype_dll )        ||
-   ! ( is_minimum_version $VERSION_FREETYPE 2.3.6                 ||
-       autogen_component freetype   $VERSION_FREETYPE )           ||
-   ! build_component   freetype   $VERSION_FREETYPE               ||
+   ! build_component   freetype   $VERSION_FREETYPE               \
+     "--without-bzip2"                                            ||
    ! free_component    freetype   $VERSION_FREETYPE "freetype"
 then
   log_error "Freetype build failed"
@@ -629,6 +643,7 @@ if ! unpack_component  fontconfig $VERSION_FONTCONFIG               ||
    ! free_component    pango      $VERSION_PANGO "pango"          ||
    ! unpack_component  atk        $VERSION_ATK                    ||
    ! ( is_smaller_version $VERSION_ATK     1.24.0  ||
+       is_minimum_version $VERSION_ATK     2.2.0   ||
        patch_src          atk-$VERSION_ATK atk_def    )           ||
    ! autogen_component atk        $VERSION_ATK                    \
      "libtoolize aclocal automake autoconf"                       ||
@@ -642,7 +657,11 @@ fi
 if ! ( is_smaller_version $VERSION_GTK2 2.22.0 ||
        ( unpack_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
          ( is_smaller_version $VERSION_GTK2 2.24.0 ||
+           is_greater_version $VERSION_GDK_PIXBUF 2.24.0 ||
            patch_src gdk-pixbuf-$VERSION_GDK_PIXBUF gdkpixbuf_gdiplusdef ) &&
+         ( ! cmp_versions $VERSION_GDK_PIXBUF 2.24.1 ||
+           patch_src gdk-pixbuf-$VERSION_GDK_PIXBUF gdkpixbuf_animiterinit) &&
+         autogen_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
          build_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
          free_component  gdk-pixbuf $VERSION_GDK_PIXBUF "gdk-pixbuf")) ||
    ! unpack_component  gtk2       $VERSION_GTK2                   ||
@@ -661,7 +680,7 @@ if ! ( is_smaller_version $VERSION_GTK2 2.22.0 ||
    ! free_component   gtk+        $VERSION_GTK2 "gtk2"            ||
    ! unpack_component gtk3        $VERSION_GTK3                   ||
    ! rm -f $CROSSER_SRCDIR/gtk+-$VERSION_GTK3/gdk/gdkconfig.h     ||
-   ! ( is_smaller_version "$VERSION_GTK3" 3.0.0 ||
+   ! ( is_minimum_version "$VERSION_GTK3" 3.2.0 ||
        patch_src gtk+-$VERSION_GTK3 gtk3_marshalers )             ||
    ! build_component_full gtk3 gtk+ $VERSION_GTK3                 ||
    ! free_component   gtk+        $VERSION_GTK3 "gtk3"            ||
@@ -708,7 +727,7 @@ if ! unpack_component  SDL        $VERSION_SDL          ||
    ! free_component    SDL_image  $VERSION_SDL_IMAGE "SDL_image" ||
    ! unpack_component  SDL_mixer  $VERSION_SDL_MIXER    ||
    ! build_component   SDL_mixer  $VERSION_SDL_MIXER    \
-     "--disable-music-mp3 --disable-smpegtest"          ||
+     "--disable-music-mp3 --disable-smpegtest --disable-music-mod" ||
    ! free_component    SDL_mixer  $VERSION_SDL_MIXER "SDL_mixer"
 then
   log_error "SDL stack build failed"
