@@ -2,7 +2,7 @@
 
 # download_packets.sh: Source package downloader
 #
-# (c) 2008-2011 Marko Lindqvist
+# (c) 2008-2012 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -267,6 +267,7 @@ if test "x$HELP_RETURN" != "x" ; then
   echo "  - xorg"
   echo "  - gtk"
   echo "  - sdl"
+  echo "  - comp"
   echo "  - win"
 
   exit $HELP_RETURN
@@ -371,10 +372,6 @@ if test "x$MIRROR_DEB" = "x" ; then
   MIRROR_DEB="http://ftp.debian.org/debian"
 fi
 
-if test "x$MIRROR_CUPS" = "x" ; then
-  MIRROR_CUPS="http://ftp.easysw.com/pub"
-fi
-
 if test "x$MIRROR_XORG" = "x" ; then
   MIRROR_XORG="http://xorg.freedesktop.org/releases"
 fi
@@ -396,23 +393,29 @@ then
     gtk-doc)     VERSION_GTK_DOC=$VERSION_SELECTED ;;
     atk)         VERSION_ATK=$VERSION_SELECTED ;;
     gcc)         VERSION_GCC=$VERSION_SELECTED ;;
-    cups)        VERSION_CUPS=$VERSION_SELECTED ;;
     readline)    VERSION_READLINE=$VERSION_SELECTED
                  PATCHES_READLINE=$PATCHES_SELECTED ;;
+    autoconf)    VERSION_AUTOCONF=$VERSION_SELECTED ;;
     automake)    VERSION_AUTOMAKE=$VERSION_SELECTED ;;
     libtool)     VERSION_LIBTOOL=$VERSION_SELECTED ;;
     mpfr)        VERSION_MPFR=$VERSION_SELECTED ;;
     Python3)     VERSION_PYTHON3=$VERSION_SELECTED ;;
     Python2)     VERSION_PYTHON2=$VERSION_SELECTED ;;
     jpeg)        VERSION_JPEG=$VERSION_SELECTED ;;
+    util-macros) VERSION_XORG_UMACROS=$VERSION_SELECTED ;;
     xproto)      VERSION_XORG_XPROTO=$VERSION_SELECTED ;;
     xextproto)   VERSION_XORG_XEXTPROTO=$VERSION_SELECTED ;;
     xtrans)      VERSION_XORG_XTRANS=$VERSION_SELECTED ;;
+    kbproto)     VERSION_XORG_KBPROTO=$VERSION_SELECTED ;;
+    inputproto)  VERSION_XORG_INPUTPROTO=$VERSION_SELECTED ;;
     libXau)      VERSION_XORG_LIBXAU=$VERSION_SELECTED ;;
     libX11)      VERSION_XORG_LIBX11=$VERSION_SELECTED ;;
+    libXext)     VERSION_XORG_LIBXEXT=$VERSION_SELECTED ;;
     linux)       VERSION_KERNEL=$VERSION_SELECTED ;;
+    sqlite)      VERSION_SQLITE=$VERSION_SELECTED ;;
   esac
 fi
+
 GLIB_DIR="$(echo $VERSION_GLIB | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 PANGO_DIR="$(echo $VERSION_PANGO | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GDK_PB_DIR="$(echo $VERSION_GDK_PIXBUF | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
@@ -424,6 +427,11 @@ ATK_DIR="$(echo $VERSION_ATK | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -
 
 READLINE_SHORT="$(echo $VERSION_READLINE | sed 's/\.//g')"
 
+case "x$VERSION_XORG_UMACROS" in
+  x1.3.0)  VERSION_XORG=X11R7.5 ;;
+  x1.11.0) VERSION_XORG=X11R7.6 ;;
+esac
+
 case "x$VERSION_XORG_XPROTO" in
   x) ;;
   x7.0.13) VERSION_XORG=X11R7.4 ;;
@@ -445,6 +453,18 @@ case "x$VERSION_XORG_XTRANS" in
   x1.2.6) VERSION_XORG=X11R7.6 ;;
 esac
 
+case "x$VERSION_XORG_KBPROTO" in
+  x) ;;
+  x1.0.4) VERSION_XORG=X11R7.5 ;;
+  x1.0.5) VERSION_XORG=X11R7.6 ;;
+esac
+
+case "x$VERSION_XORG_INPUTPROTO" in
+  x) ;;
+  x2.0)   VERSION_XORG=X11R7.5 ;;
+  x2.0.1) VERSION_XORG=X11R7.6 ;;
+esac
+
 case "x$VERSION_XORG_LIBXAU" in
   x) ;;
   x1.0.4) VERSION_XORG=X11R7.4 ;;
@@ -459,11 +479,29 @@ case "x$VERSION_XORG_LIBX11" in
   x1.4.0) VERSION_XORG=X11R7.6 ;;
 esac
 
-if is_minimum_version $VERSION_AUTOMAKE 1.6.1
+case "x$VERSION_XORG_LIBXEXT" in
+  x) ;;
+  x1.1.1) VERSION_XORG=X11R7.5 ;;
+  x1.2.0) VERSION_XORG=X11R7.6 ;;
+esac
+
+if cmp_versions $VERSION_SQLITE 3.7.10
 then
-  AUTOMAKE_PACK="tar.bz2"
+  SQL_VERSTR="3071000"
+fi
+
+if is_minimum_version $VERSION_CAIRO 1.12.2
+then
+  CAIRO_PACK="tar.xz"
 else
-  AUTOMAKE_PACK="tar.gz"
+  CAIRO_PACK="tar.gz"
+fi
+
+if is_minimum_version $VERSION_AUTOMAKE 1.11.3
+then
+  AUTOMAKE_PACK="tar.xz"
+else
+  AUTOMAKE_PACK="tar.bz2"
 fi
 
 if is_minimum_version $VERSION_LIBTOOL 2.2.6a
@@ -473,16 +511,80 @@ else
   LIBTOOL_PACK="tar.bz2"
 fi
 
-KERNEL_DIR="v$(echo $VERSION_KERNEL | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+if is_minimum_version $VERSION_AUTOCONF 2.68b
+then
+  AUTOCONF_PACK="tar.xz"
+else
+  AUTOCONF_PACK="tar.bz2"
+fi
 
-download_needed "$MIRROR_GNU/libtool/"  "libtool"  "$VERSION_LIBTOOL"  $LIBTOOL_PACK
+if is_minimum_version $VERSION_MPFR 3.1.0
+then
+  MPFR_PACK="tar.xz"
+else
+  MPFR_PACK="tar.bz2"
+fi
 
-RET="$?"  
+# While there's earlier .xz packaged glib version available,
+# earlier crosser versions fetched them as .bz2. In case such
+# an .bz2 package already exist, we don't want to download .xz
+# of the same version.
+if is_minimum_version $VERSION_GLIB 2.30.3
+then
+  GLIB_PACK="tar.xz"
+else
+  GLIB_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_ATK 2.4.0
+then
+  ATK_PACK="tar.xz"
+else
+  ATK_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_PANGO 1.30.0
+then
+  PANGO_PACK="tar.xz"
+else
+  PANGO_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_GTK2 2.24.9
+then
+  GTK2_PACK="tar.xz"
+else
+  GTK2_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_GTK3 3.2.0
+then
+  GTK3_PACK="tar.xz"
+else
+  GTK3_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_GDK_PIXBUF 2.24.0
+then
+  GDK_PB_PACK="tar.xz"
+else
+  GDK_PB_PACK="tar.bz2"
+fi
+
+if is_minimum_version $VERSION_KERNEL 3.0
+then
+  KERNEL_DIR="v3.0"
+else
+  KERNEL_DIR="v$(echo $VERSION_KERNEL | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+fi
+
+download_needed "$MIRROR_GNU/libtool/"  "libtool"  "$VERSION_LIBTOOL"  "$LIBTOOL_PACK"
+RET="$?"
 download_needed "$MIRROR_GNU/binutils/" "binutils" "$VERSION_BINUTILS" "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_GNU/gawk/"     "gawk"     "$VERSION_GAWK"     "tar.bz2"
 RET="$RET $?"
-download_needed "$MIRROR_GNU/autoconf/" "autoconf" "$VERSION_AUTOCONF" "tar.bz2"
+download_needed "$MIRROR_GNU/autoconf/" "autoconf" "$VERSION_AUTOCONF" "$AUTOCONF_PACK"
 RET="$RET $?"
 download_needed "$MIRROR_GNU/automake/" "automake" "$VERSION_AUTOMAKE" "$AUTOMAKE_PACK"
 RET="$RET $?"
@@ -506,7 +608,7 @@ download_needed "$MIRROR_SOURCEWARE/pub/newlib/"        "newlib" "$VERSION_NEWLI
 RET="$RET $?"
 download_needed "http://ftp.sunet.se/pub/gnu/gmp/"      "gmp"    "$VERSION_GMP"    "tar.bz2"
 RET="$RET $?"
-download_needed "http://www.mpfr.org/mpfr-$VERSION_MPFR/" "mpfr"   "$VERSION_MPFR"   "tar.bz2"
+download_needed "http://www.mpfr.org/mpfr-$VERSION_MPFR/" "mpfr"   "$VERSION_MPFR"   "$MPFR_PACK"
 RET="$RET $?"
 download_needed "http://www.multiprecision.org/mpc/download/" "mpc" "$VERSION_MPC"   "tar.gz"
 RET="$RET $?"
@@ -520,6 +622,8 @@ download_needed "$MIRROR_DEB/pool/main/z/zlib/"         "zlib"       "$VERSION_Z
 RET="$RET $?"
 download_needed "http://www.bzip.org/$VERSION_BZIP2/"   "bzip2"      "$VERSION_BZIP2"      "tar.gz"
 RET="$RET $?"
+download_needed "http://tukaani.org/xz/"                "xz"         "$VERSION_XZ"         "tar.xz"
+RET="$RET $?"
 download_needed "$MIRROR_GNU/readline/"                 "readline"   "$VERSION_READLINE"   "tar.gz"
 RET="$RET $?"
 download_patches "$MIRROR_GNU/readline/readline-$VERSION_READLINE-patches/" \
@@ -528,7 +632,7 @@ download_patches "$MIRROR_GNU/readline/readline-$VERSION_READLINE-patches/" \
 RET="$RET $?"
 download_needed "$MIRROR_GNU/gettext/"                  "gettext"    "$VERSION_GETTEXT"    "tar.gz"
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/glib/$GLIB_DIR/" "glib"       "$VERSION_GLIB"       "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/glib/$GLIB_DIR/" "glib"       "$VERSION_GLIB"       "$GLIB_PACK"
 RET="$RET $?"
 download_needed "http://www.ijg.org/files/"             "jpeg"       "jpegsrc.v${VERSION_JPEG}.tar.gz"
 RET="$RET $?"
@@ -546,24 +650,22 @@ download_needed "ftp://sourceware.org/pub/libffi/"      "libffi"     "$VERSION_F
 RET="$RET $?"
 download_needed "http://cairographics.org/releases/"    "pixman"     "$VERSION_PIXMAN"     "tar.gz"
 RET="$RET $?"
-download_needed "http://cairographics.org/releases/"    "cairo"      "$VERSION_CAIRO"      "tar.gz"
+download_needed "http://cairographics.org/releases/"    "cairo"      "$VERSION_CAIRO"      "$CAIRO_PACK"
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/pango/$PANGO_DIR/" "pango"    "$VERSION_PANGO"      "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/pango/$PANGO_DIR/" "pango"    "$VERSION_PANGO"      "$PANGO_PACK"
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/atk/$ATK_DIR/"   "atk"        "$VERSION_ATK"        "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/atk/$ATK_DIR/"   "atk"        "$VERSION_ATK"        "$ATK_PACK"
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk-doc/$GTK_DOC_DIR/" "gtk-doc" "$VERSION_GTK_DOC" "tar.bz2"
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/gdk-pixbuf/$GDK_PB_DIR/" "gdk-pixbuf" "$VERSION_GDK_PIXBUF"  "tar.bz2" 
+download_needed "$MIRROR_GNOME/sources/gdk-pixbuf/$GDK_PB_DIR/" "gdk-pixbuf" "$VERSION_GDK_PIXBUF"  "$GDK_PB_PACK" 
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/gtk+/$GTK2_DIR/" "gtk2"       "$VERSION_GTK2"        "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/gtk+/$GTK2_DIR/" "gtk2"       "$VERSION_GTK2"        "$GTK2_PACK"
 RET="$RET $?"
-download_needed "$MIRROR_GNOME/sources/gtk+/$GTK3_DIR/" "gtk3"       "$VERSION_GTK3"        "tar.bz2"
+download_needed "$MIRROR_GNOME/sources/gtk+/$GTK3_DIR/" "gtk3"       "$VERSION_GTK3"        "$GTK3_PACK"
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk-engines/$GTK_ENG_DIR/"  "gtk-engines" "$VERSION_GTK_ENG" "tar.bz2"
 RET="$RET $?"
-download_needed "$MIRROR_CUPS/cups/$VERSION_CUPS/"     "cups"       "cups-$VERSION_CUPS-source.tar.bz2"
-RET="$RET $?"
 download_needed "http://www.libsdl.org/release/"        "SDL"        "$VERSION_SDL"        "tar.gz"
 RET="$RET $?"
 download_needed "http://www.libsdl.org/projects/SDL_image/release/" "SDL_image"  "$VERSION_SDL_IMAGE"  "tar.gz"
@@ -580,20 +682,30 @@ download_needed "http://xcb.freedesktop.org/dist/"           "xcb-proto" "$VERSI
 RET="$RET $?"
 download_needed "http://xcb.freedesktop.org/dist/"           "libxcb"    "$VERSION_LIBXCB"         "tar.bz2"
 RET="$RET $?"
+download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "util-macros"    "$VERSION_XORG_UMACROS"    "tar.bz2"
+RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xproto"    "$VERSION_XORG_XPROTO"    "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xextproto" "$VERSION_XORG_XEXTPROTO" "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xtrans"    "$VERSION_XORG_XTRANS" "tar.bz2"
 RET="$RET $?"
+download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "kbproto"   "$VERSION_XORG_KBPROTO" "tar.bz2"
+RET="$RET $?"
+download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "inputproto" "$VERSION_XORG_INPUTPROTO" "tar.bz2"
+RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "libXau"    "$VERSION_XORG_LIBXAU" "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "libX11"    "$VERSION_XORG_LIBX11" "tar.bz2"
 RET="$RET $?"
+download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "libXext"   "$VERSION_XORG_LIBXEXT" "tar.bz2"
+RET="$RET $?"
 download_needed "ftp://ftp.gnupg.org/gcrypt/libgpg-error/"   "libgpg-error" "$VERSION_GPGERROR" "tar.bz2"
 RET="$RET $?"
 download_needed "ftp://ftp.gnupg.org/gcrypt/libgcrypt/"      "libgcrypt" "$VERSION_LIBGCRYPT"   "tar.bz2"
 RET="$RET $?"
+download_needed "http://www.sqlite.com/" "sqlite" "autoconf-${SQL_VERSTR}" "tar.gz"
+RET="$RET $?"
 
 for VALUE in $RET
 do
