@@ -2,7 +2,7 @@
 
 # helpers.sh: Functions for Crosser
 #
-# (c) 2008-2011 Marko Lindqvist
+# (c) 2008-2012 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -29,6 +29,10 @@ if test "x$LOGLEVEL_FILE" = "x" ; then
   LOGLEVEL_FILE=4
 fi
 
+if which cvercmp >/dev/null 2>&1 ; then
+  CROSSER_CVERCMP=yes
+fi
+
 # Start new logfiles
 #
 log_init() {
@@ -248,6 +252,12 @@ unpack_component() {
       log_error "Unpacking $NAME_BASE.tar.gz failed"
       return 1
     fi
+  elif test -e "$PACKETDIR/$NAME_BASE.tar.xz" ; then
+    if ! tar xJf "$PACKETDIR/$NAME_BASE.tar.xz" -C "$CROSSER_SRCDIR/$3"
+    then
+      log_error "Unpacking $NAME_BASE.tar.xz failed"
+      return 1
+    fi
   elif test -e "$PACKETDIR/$NAME_BASE.tgz" ; then
     if ! tar xzf "$PACKETDIR/$NAME_BASE.tgz" -C "$CROSSER_SRCDIR/$3"
     then
@@ -478,6 +488,19 @@ tokenize_version() {
 # 1  - $1 is greater than $2
 # 2  - $2 is greater than $1
 cmp_versions() {
+  if test "x$CROSSER_CVERCMP" = "xyes"
+  then
+    if cvercmp "$1" equal "$2" > /dev/null
+    then
+      return 0
+    fi
+    if cvercmp "$1" greater "$2" > /dev/null
+    then
+      return 1
+    fi
+    return 2
+  fi
+
   VER1PARTS="$(tokenize_version $1)"
   VER2PARTS="$(tokenize_version $2)"
 
@@ -575,6 +598,12 @@ cmp_versions() {
 # $1 - Version number
 # $2 - Comparison version
 is_minimum_version() {
+  if test "x$CROSSER_CVERCMP" = "xyes"
+  then
+    cvercmp "$1" min "$2" > /dev/null
+    return $?
+  fi
+
   cmp_versions "$1" "$2"
 
   if test $? -eq 2
@@ -588,6 +617,12 @@ is_minimum_version() {
 # $1 - Version number
 # $2 - Comparison version
 is_max_version() {
+  if test "x$CROSSER_CVERCMP" = "xyes"
+  then
+    cvercmp "$1" max "$2" > /dev/null
+    return $?
+  fi
+
   cmp_versions "$1" "$2"
 
   if test $? -eq 1
@@ -601,6 +636,12 @@ is_max_version() {
 # $1 - Version number
 # $2 - Comparison version
 is_smaller_version() {
+  if test "x$CROSSER_CVERCMP" = "xyes"
+  then
+    cvercmp "$1" lesser "$2" > /dev/null
+    return $?
+  fi
+
   cmp_versions "$1" "$2"
 
   if test $? -ne 2
@@ -614,6 +655,12 @@ is_smaller_version() {
 # $1 - Version number
 # $2 - Comparison version
 is_greater_version() {
+  if test "x$CROSSER_CVERCMP" = "xyes"
+  then
+    cvercmp "$1" greater "$2" > /dev/null
+    return $?
+  fi
+
   cmp_versions "$1" "$2"
 
   if test $? -ne 1
